﻿using System;
using System.Collections.Generic;
using System.Drawing;
using System.Globalization;
using System.IO;
using System.Linq;
using System.Net;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace PristaniNaMesici
{
    public class Pristani : IDisposable
    {
        public int ID { get; private set; }
        public string Address { get; private set; }
        public Modul Modul { get; private set; }
        public float FinishL { get; private set; }
        public float FinishR { get; private set; }

        public bool AutoPilot { get; }

        public float Zoom
        {
            get
            {
                float x = Control.Width / 200;
                float y = Control.Height / Math.Max((Modul.Position.Y + 10), 200);
                return Math.Min(x, y);
            }
        }
        /// <summary>v jakometrech</summary>
        public PointF Camera
        {
            get
            {
                float x = FinishL + ((FinishR - FinishL) / 2) - (Control.Width / 2 / Zoom);
                return new PointF(x, 0);
            }
        }

        public event Action<string> End;

        public PictureBox Control { get; }
        Timer timer = new Timer()
        {
            Interval = 20
        };

        public Pristani(string address, PictureBox control, bool autoPilot = false)
        {
            AutoPilot = autoPilot;
            Control = control;
            Address = address;
            Modul = new Modul();
            Update(address + "/start");

            timer.Tick += (object sender, EventArgs e) =>
            {
                if (AutoPilot) AutoPilotControl();
                else Update($"{Address}/stav?id={ID}");
            };
            timer.Start(); 
        }
        
        protected void Update(string address)
        {
            WebRequest request = WebRequest.Create(address);
            request.Method = "GET";

            using (WebResponse response = request.GetResponse())
            {
                using (StreamReader stream = new StreamReader(response.GetResponseStream(), Encoding.ASCII))
                {
                    Set(stream);
                }
            }
            Draw();
        }
        void Set(StreamReader stream)
        {
            //destina .
            NumberFormatInfo format = new NumberFormatInfo()
            {
                NumberDecimalSeparator = "."
            };

            //radky
            //1.
            string s = stream.ReadLine();
            int id;
            if (int.TryParse(s, out id)) ID = id;
            else
            {
                // konec
                timer.Stop();
                End?.Invoke(s);
                return;
            }
            //2. pozice
            string[] sposition = stream.ReadLine().Split(' ');
            Modul.Position = new PointF()
            {
                X = float.Parse(sposition[0], format),
                Y = float.Parse(sposition[1], format)
            };
            //3. rychlost
            string[] sspeed = stream.ReadLine().Split(' ');
            Modul.Speed = new PointF()
            {
                X = float.Parse(sspeed[0], format),
                Y = float.Parse(sspeed[1], format)
            };
            //4. palivo
            Modul.Palivo = float.Parse(stream.ReadLine(), format);
            //5. pristavci plocha
            string[] sfinish = stream.ReadLine().Split(' ');
            FinishL = float.Parse(sfinish[0], format);
            FinishR = float.Parse(sfinish[1], format);
        }
        public void SetMotor(bool up, bool left, bool right)
        {
            Update($"{Address}/stav?id={ID}&nahoru={(up ? 1 : 0)}&doleva={(left ? 1 : 0)}&doprava={(right ? 1 : 0)}");
        }
        public void Dispose()
        {
            timer.Dispose();
        }
        
        void Draw()
        {
            float zoom = Zoom;
            PointF camera = Camera;

            Bitmap bitmap = new Bitmap(Control.Width, Control.Height);
            using (Graphics g = Graphics.FromImage(bitmap))
            {
                g.Clear(Color.DarkGray);

                //Modul
                g.DrawImage(Modul.Picture, Modul.GetPosOnScreen(zoom, camera, Control.Height));
                using (Pen pen = new Pen(Color.Black, 3))
                {
                    //Finish
                    g.DrawLine(pen, zoom * (FinishL - camera.X), bitmap.Height - 1, zoom * (FinishR - camera.X), bitmap.Height - 1);
                }

                Modul.DrawPalivo(g, Control.ClientSize);
                Modul.DrawSpeed(g);
            }
            if (Control.Image != null) Control.Image.Dispose();
            Control.Image = bitmap;
            Control.Invalidate();
        }

        void AutoPilotControl()
        {
            if (Modul.Speed.Y < -4) SetMotor(true, false, false);
            //if(Modul.Position.X < FinishL)
            else
                SetMotor(false, false, false);
        }
    }
}
